package com.xy.redis;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.xy.collections.list.JArrayList;
import com.xy.collections.list.JList;
import com.xy.collections.map.JMap;
import com.xy.entity.*;
import com.xy.mapper.SysDeptMapper;
import com.xy.mapper.SysMenuMapper;
import com.xy.mapper.SysRoleMapper;
import com.xy.mapper.SysSystemMapper;
import com.xy.utils.AuthorizeUtils;
import com.xy.utils.Emptys;
import com.xy.utils.RedisService;
import com.xy.utils.SpringBeanUtils;
import icu.mhb.mybatisplus.plugln.core.JoinLambdaWrapper;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 权限系统redis工具类
 */
@Component
public class RedisCache extends AuthorizeUtils {

    /**
     * 缓存业务系统权限到redis
     *
     * @param loginId 登录id
     */
    public static void workCache(Long loginId) {
        SysSystemMapper sysSystemMapper = SpringBeanUtils.getBean(SysSystemMapper.class);
        SysRoleMapper sysRoleMapper = SpringBeanUtils.getBean(SysRoleMapper.class);
        SysDeptMapper sysDeptMapper = SpringBeanUtils.getBean(SysDeptMapper.class);
        SysMenuMapper sysMenuMapper = SpringBeanUtils.getBean(SysMenuMapper.class);
        RedisService<List<AuthorizeUtils.CacheEntity>> roleMenuRedisService = getRedisService();
        //获取登录人所有系统
        JoinLambdaWrapper<SysSystem> sysSystemsJoinLambdaWrapper = new JoinLambdaWrapper<>(SysSystem.class)
                .eq(SysSystem::getStatus, true)
                .innerJoin(SysUserRole.class, SysUserRole::getSysId, SysSystem::getId)
                .eq(SysUserRole::getWorkUserId, loginId)
                .end();
        List<SysSystem> sysSystems = sysSystemMapper.joinSelectList(sysSystemsJoinLambdaWrapper, SysSystem.class);
        if (!Emptys.check(sysSystems)) {
            return;
        }
        //获取登录人所有角色
        JoinLambdaWrapper<SysRole> sysRoleJoinLambdaWrapper = new JoinLambdaWrapper<>(SysRole.class)
                .eq(SysRole::getStatus, true)
                .innerJoin(SysUserRole.class, SysUserRole::getRoleId, SysRole::getId)
                .eq(SysUserRole::getWorkUserId, loginId)
                .end();
        List<SysRole> sysRoles = sysRoleMapper.joinSelectList(sysRoleJoinLambdaWrapper, SysRole.class);
        JList<Long> sysRoleIds = new JArrayList<>(sysRoles).getProperty(SysRole::getId);
        if (sysRoleIds.size() == 0) {
            return;
        }
        //获取登录人所有部门
        JoinLambdaWrapper<SysDept> sysDeptJoinLambdaWrapper = new JoinLambdaWrapper<>(SysDept.class)
                .eq(SysDept::getStatus, true)
                .innerJoin(SysUserDept.class, SysUserDept::getDeptId, SysUserDept::getId)
                .eq(SysUserDept::getWorkUserId, loginId)
                .end();
        List<SysDept> sysDepts = sysDeptMapper.joinSelectList(sysDeptJoinLambdaWrapper, SysDept.class);
        JList<Long> sysDeptsIds = new JArrayList<>(sysDepts).getProperty(SysDept::getId);
        //获取登录人所有部门角色
        if (sysDeptsIds.size() > 0) {
            sysRoleJoinLambdaWrapper = new JoinLambdaWrapper<>(SysRole.class)
                    .eq(SysRole::getStatus, true)
                    .innerJoin(SysDeptRole.class, SysDeptRole::getRoleId, SysRole::getId)
                    .in(SysDeptRole::getDeptId, sysDeptsIds)
                    .end();
            List<SysRole> sysRoles2 = sysRoleMapper.joinSelectList(sysRoleJoinLambdaWrapper, SysRole.class);
            JList<Long> sysRoleIds2 = new JArrayList<>(sysRoles2).getProperty(SysRole::getId);
            if (sysRoleIds2.size() > 0) {
                sysRoleIds.setAll(sysRoleIds2)
                        .comparing();
                sysRoles = new JArrayList<>(sysRoles2).setAll(sysRoles);
            }
        }
        //获取登录人所有角色菜单
        JoinLambdaWrapper<SysMenu> sysMenuJoinLambdaWrapper = new JoinLambdaWrapper<>(SysMenu.class)
                .eq(SysMenu::getStatus, true)
                .innerJoin(SysRoleMenu.class, SysRoleMenu::getMenuId, SysMenu::getId)
                .selectAs(cb -> cb
                        .add(SysRoleMenu::getSelectIsAdmin)
                        .add(SysRoleMenu::getRoleId)
                )
                .in(SysRoleMenu::getRoleId, sysRoleIds)
                .end();
        List<SysMenuJoinSysRoleMenu> sysMenus = sysMenuMapper.joinSelectList(sysMenuJoinLambdaWrapper, SysMenuJoinSysRoleMenu.class);
        JMap<Long, List<SysMenuJoinSysRoleMenu>> sysMenuMaps = new JArrayList<>(sysMenus).toMap(SysMenuJoinSysRoleMenu::getRoleId).group();
        for (SysSystem sysSystem : sysSystems) {
            List<AuthorizeUtils.CacheEntity> cacheEntitys = new ArrayList<>();
            JList<SysRole> list = new JArrayList<>(sysRoles).filter().eq(SysRole::getSysId, sysSystem.getId()).list();
            list.forEach(sysRole -> {
                AuthorizeUtils.CacheEntity cacheEntity = new AuthorizeUtils.CacheEntity()
                        .setRoleId(sysRole.getId())
                        .setRoleCode(sysRole.getCode())
                        .setList(sysMenuMaps.get(sysRole.getId()));
                cacheEntitys.add(cacheEntity);
            });
            roleMenuRedisService.setMap(getKey(loginId), sysSystem.getId().toString(), cacheEntitys);
        }
    }

    /**
     * 判断数据查询权限
     *
     * @param lambdaQueryWrapper 查询构造器
     * @param function           普通查询权限时，条件lambda引用字段
     * @return true=管理员数据查询权限 false=普通查询权限
     */
    public static <T> boolean getDataAuth(Long systemId, LambdaQueryWrapper<T> lambdaQueryWrapper, SFunction<T, ?> function) {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        boolean dataAuth = authByData(systemId, request.getRequestURI());
        if (!dataAuth) {
            lambdaQueryWrapper.eq(function, getLoginId(Long.class));
        }
        return false;
    }

    /**
     * 检验系统ID是否属于当前登录人
     *
     * @param systemId 系统id
     * @return
     */
    public static Boolean checkSystemId(Long systemId) {
        Map<String, List<CacheEntity>> map = getRedisService().getMap(getKey(getLoginId(Long.class)));
        return map.containsKey(systemId.toString());
    }
}
